@@ -640,13 +640,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
      */
     tls_check_version();
 
-    /*
-     * Initialize the OpenSSL library, possibly loading its configuration
-     * file.
-     */
-    if (tls_library_init() == 0)
-	return (0);
-
     /*
      * Create an application data index for SSL objects, so that we can
      * attach TLScontext information; this information is needed inside
@@ -720,15 +713,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
     }
     tls_dane_digest_init(client_ctx, fpt_alg);
 
-    /*
-     * Presently we use TLS only with SMTP where truncation attacks are not
-     * possible as a result of application framing.  If we ever use TLS in
-     * some other application protocol where truncation could be relevant,
-     * we'd need to disable truncation detection conditionally, or explicitly
-     * clear the option in that code path.
-     */
-    off |= SSL_OP_IGNORE_UNEXPECTED_EOF;
-
     /*
      * Protocol selection is destination dependent, so we delay the protocol
      * selection options to the per-session SSL object.
@@ -795,10 +779,9 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
     /*
      * With OpenSSL 1.0.2 and later the client EECDH curve list becomes
      * configurable with the preferred curve negotiated via the supported
-     * curves extension.  With OpenSSL 3.0 and TLS 1.3, the same applies
-     * to the FFDHE groups which become part of a unified "groups" list.
+     * curves extension.
      */
-    tls_auto_groups(client_ctx, var_tls_eecdh_auto, var_tls_ffdhe_auto);
+    tls_auto_eecdh_curves(client_ctx, var_tls_eecdh_auto);
 
     /*
      * Finally, the setup for the server certificate checking, done "by the
